#######################################################
# 画像の回転
#
# Shin.Onda 2023/08//26 all rights reserved.
#
#######################################################
import re
import numpy as np
import random
import cv2
from createcanvas import CreateCanvas

class ImageRotate :

    ######################################
    # イメージ回転
    # パラメータ
    #   paraImg   = イメージ
    #   imgBack   = 背景('R':乱数)
    #   startAngle= 回転開始角度(右)
    #   endAngle  = 回転終了角度(左)
    #   step      = 間隔(>1)
    # 戻り値 イメージリスト
    ######################################
    def rotate(self, paramImg, imgBack='R', startAngle=60, endAngle=60, step=10) :
        # 画像パラメータ
        orgH, orgW, orgC = paramImg.shape

        # 背景を含めた回転用に正方形のキャンバスに背景を設定
        # 背景を描画したキャンバスに元画像を張り付ける
        if orgW > orgH :
            baseSize = orgW
        else :
            baseSize = orgH
        baseSize = int(baseSize * 1.5)
        canvas = CreateCanvas.create(baseSize, baseSize, imgBack)
        sx = int((baseSize - orgW) / 2.0)
        ex = int(sx + orgW)
        sy = int((baseSize - orgH) / 2.0)
        ey = int(sy + orgH)
        canvas[sy:ey, sx:ex] = paramImg

        # 回転のループ
        imgList = []
        # 開始位置(右角度の場合はマイナスからスタート)
        locStart = - startAngle
        # 角度計算
        delta = (startAngle + endAngle) / (step-1)
        for i in range(0, step) :
            # 合わせたサイズ変更
            deg = locStart + (delta * i)
            print(deg)

            #画像の回転行列
            rotmatrix = cv2.getRotationMatrix2D((baseSize/2, baseSize/2), deg, 1)

            # アフィン変換適用
            newCanvas = cv2.warpAffine(canvas, rotmatrix, (baseSize, baseSize))

            # 元のサイズに戻して書き込む
            newImg = newCanvas[sy:ey, sx:ex]

            # 戻り値リストに追加
            imgList.append(newImg)
        
        return imgList
            
