#######################################################
# 画像のシフト
#
# Shin.Onda 2023/08//26 all rights reserved.
#
#######################################################
import re
import numpy as np
import random
import cv2
from createcanvas import CreateCanvas

class ImageShift :

    ######################################
    # イメージ水平移動
    # パラメータ
    #   paraImg = イメージ
    #   imgBack = 背景('R':乱数)
    #   leftShift = 左シフト　開始％
    #   rightShift= 右シフト　終了％
    #   magStep  = 間隔(>1)
    # 戻り値 イメージリスト
    ######################################
    def horShift(self, paramImg, imgBack='R', leftShift=50, rightShift=50, step=10) :
        # 画像パラメータ
        orgH, orgW, orgC = paramImg.shape

        # シフトのループ
        imgList = []
        # 開始位置(左の場合はマイナスからスタート)
        locStart = - orgW * (leftShift / 100.0)
        # 移動ピクセル量計算
        delta = orgW * ((rightShift + leftShift) / (step-1) / 100.0)
        for i in range(0, step) :
            # 背景を描画したキャンバス
            canvas = CreateCanvas.create(orgH, orgW, imgBack)

            # 元画像を切り取って張り付ける
            loc = int(locStart + delta * i)
            print(loc)
            if loc < 0 :
                canvas[0:orgH, 0:orgW+loc] = paramImg[0:orgH, -loc:orgW]
            else :
                canvas[0:orgH, loc:orgW] = paramImg[0:orgH, 0:orgW-loc]
            # 戻り値リストに追加
            imgList.append(canvas)
        
        return imgList

    ######################################
    # イメージ垂直移動
    # パラメータ
    #   paraImg = イメージ
    #   imgBack = 背景('R':乱数)
    #   topShift   = 上シフト　開始％
    #   bottomShift= 下シフト　終了％
    #   magStep  = 間隔(>1)
    # 戻り値 イメージリスト
    ######################################
    def verShift(self, paramImg, imgBack='R', topShift=50, bottomShift=50, step=10) :
        # 画像パラメータ
        orgH, orgW, orgC = paramImg.shape

        # シフトのループ
        imgList = []
        # 開始位置(上の場合はマイナスからスタート)
        locStart = - orgH * (topShift / 100.0)
        # 移動ピクセル量計算
        delta = orgH * ((bottomShift + topShift) / (step-1) / 100.0)
        for i in range(0, step) :
            # 背景を描画したキャンバス
            canvas = CreateCanvas.create(orgH, orgW, imgBack)

            # 元画像を切り取って張り付ける
            loc = int(locStart + delta * i)
            print(loc)
            if loc < 0 :
                canvas[0:orgH+loc, 0:orgW] = paramImg[-loc:orgH, 0:orgW]
            else :
                canvas[loc:orgH, 0:orgW] = paramImg[0:orgH-loc, 0:orgW]
            # 戻り値リストに追加
            imgList.append(canvas)
        
        return imgList


