#######################################################
# 画像のスキュー
#
# Shin.Onda 2023/08//26 all rights reserved.
#
#######################################################
import numpy as np
import math
import cv2
from createcanvas import CreateCanvas

class ImageSkew :

    ######################################
    # スキュー(左右)
    # パラメータ
    #   paraImg   = イメージ
    #   imgBack   = 背景('R':乱数、'W':白、'B':黒)
    #   startAngle= 開始角度(左斜め)
    #   endAngle  = 終了角度(右斜め)
    #   magStep   = 間隔(>1)
    # 戻り値 イメージリスト
    ######################################
    def horSkew(self, paramImg, imgBack='R', startAngle=60, endAngle=60, step=10) :
        # 画像パラメータ
        orgH, orgW, orgC = paramImg.shape

        # 角度のループ
        imgList = []
        # 開始位置(右角度の場合はマイナスからスタート)
        locStart = - startAngle
        # 角度計算
        delta = (startAngle + endAngle) / (step-1)
        for i in range(0, step) :
            # 合わせたサイズ変更
            deg = locStart + (delta * i)
            print(deg)

            # 変換行列
            tan = math.tan(math.radians(deg))
            if tan >= 0 :
                mat = np.array([[1, tan, 0], [0, 1, 0]], dtype=np.float32)
            else :
                mat = np.array([[1, tan, -orgH*tan], [0, 1, 0]], dtype=np.float32)

            # 背景
            newW = int(orgW + abs(orgH*tan))
            newH = orgH
            canvas = CreateCanvas.create(newH, newW, imgBack)

            # アフィン変換
            newCanvas = cv2.warpAffine(paramImg, mat, (newW, newH), borderMode=cv2.BORDER_TRANSPARENT, dst=canvas)

            # 元のサイズで中央を抜き出す
            sx = int((newW - orgW) / 2.0)
            ex = sx + orgW
            newImg = newCanvas[0:orgH, sx:ex]

            # 戻り値リストに追加
            imgList.append(newImg)
        
        return imgList
            
