##################################################
# 画像変換メイン
#
# Shin.Onda 2023/08//26 all rights reserved.
#
##################################################
import os
import imagetransformmanager
import tkinter as tk
from tkinter import ttk
import tkinter.messagebox as messagebox
from functools import partial

# 画像のデフォルト初期値
defaultDir = os.path.abspath(os.path.dirname("./"))

# メインウインドウのボタン処理オブジェクト
root = tk.Tk()

###########################
# 入力値の検証
###########################
###########################
# 数値入力チェック(整数)
#   P = 入力値
# 戻り値 True 入力可、False 入力不可
###########################
def validateDegit(P) :
    ret = True

    if len(P) > 0 :
        if not P.isdecimal() :
            ret = False

    return ret

###########################
# 空白、NULL
#   value
# 戻り値 True 空白,NULL、False その他
###########################
def isEmpty(value) :
    if not value :
        return True
    return False

###########################
# 効果
#   value
# 戻り値 True 入力可、False 入力不可
###########################
def validateEffect(value) :

    if isEmpty(value) :
        tk.Tk().withdraw()
        ret = messagebox.showinfo('エラー', '効果の強度を入力してください。')
        return False

    if int(value) < 2 :
        tk.Tk().withdraw()
        ret = messagebox.showinfo('エラー', '効果の強度は２以上にしてください。')
        return False

    if int(value) > 50 :
        tk.Tk().withdraw()
        ret = messagebox.showinfo('エラー', '効果の強度は50以下にしてください。')
        return False

    return True

###########################
# ステップ（段数）
#   step
# 戻り値 True 入力可、False 入力不可
###########################
def validateStep(step) :

    if int(step) < 2 or int(step) > 100:
        tk.Tk().withdraw()
        ret = messagebox.showinfo('エラー', 'ステップは2以上100以下にしてください。')
        return False
    return True


###########################
# 回転範囲(右～左)
#   startAngle 左 (°)
#   endAngle   右 (°)
#   step   ステップ
# 戻り値 True 入力可、False 入力不可
###########################
def validateRotate(startAngle, endAngle, step) :

    if isEmpty(startAngle) or isEmpty(endAngle) or isEmpty(step) :
        tk.Tk().withdraw()
        ret = messagebox.showinfo('エラー', '回転範囲の値を入力してください。')
        return False

    if int(startAngle) > 180 or int(endAngle) > 180 :
        tk.Tk().withdraw()
        ret = messagebox.showinfo('エラー', '回転範囲の値は180°以下にしてください。')
        return False

    if int(startAngle) + int(endAngle) <= 0 :
        tk.Tk().withdraw()
        ret = messagebox.showinfo('エラー', '回転の範囲がありません。')
        return False

    if not validateStep(step) :
        return False

    return True


###########################
# スキュー範囲(右、左)
#   startAngle 左 (°)
#   endAngle   右 (°)
#   step   ステップ
# 戻り値 True 入力可、False 入力不可
###########################
def validateSkew(startAngle, endAngle, step) :

    if isEmpty(startAngle) or isEmpty(endAngle) or isEmpty(step) :
        tk.Tk().withdraw()
        ret = messagebox.showinfo('エラー', '傾斜の値を入力してください。')
        return False

    if int(startAngle) > 45 or int(endAngle) > 45 :
        tk.Tk().withdraw()
        ret = messagebox.showinfo('エラー', '傾斜の値は45°以下にしてください。')
        return False

    if int(startAngle) + int(endAngle) <= 0 :
        tk.Tk().withdraw()
        ret = messagebox.showinfo('エラー', '傾斜の範囲がありません。')
        return False

    if not validateStep(step) :
        return False

    return True

###########################
# シフト範囲(左右)
#   left   左 (%)
#   right  右 (%)
#   step   ステップ
# 戻り値 True 入力可、False 入力不可
###########################
def validateHorShift(left, right, step) :

    if isEmpty(left) or isEmpty(right) or isEmpty(step) :
        tk.Tk().withdraw()
        ret = messagebox.showinfo('エラー', '左右シフトの値を入力してください。')
        return False

    if int(left) > 50 or int(right) > 50 :
        tk.Tk().withdraw()
        ret = messagebox.showinfo('エラー', '左右シフトの値は50%以下にしてください。')
        return False

    if int(left) + int(right) <= 0 :
        tk.Tk().withdraw()
        ret = messagebox.showinfo('エラー', '左右シフトの範囲がありません。')
        return False

    if not validateStep(step) :
        return False

    return True

###########################
# シフト範囲(上下)
#   top    上 (%)
#   bottom 下 (%)
#   step   ステップ
# 戻り値 True 入力可、False 入力不可
###########################
def validateVerShift(top, bottom, step) :

    if isEmpty(top) or isEmpty(bottom) or isEmpty(step) :
        tk.Tk().withdraw()
        ret = messagebox.showinfo('エラー', '上下シフトの値を入力してください。')
        return False

    if int(top) > 50 or int(bottom) > 50 :
        tk.Tk().withdraw()
        ret = messagebox.showinfo('エラー', '上下シフトの値は50%以下にしてください。')
        return False

    if int(top) + int(bottom) <= 0 :
        tk.Tk().withdraw()
        ret = messagebox.showinfo('エラー', '上下シフトの範囲がありません。')
        return False

    if not validateStep(step) :
        return False

    return True


###########################
# 縮小拡大
#   min   最小(%)
#   max   最大(%)
#   step  ステップ
# 戻り値 True 入力可、False 入力不可
###########################
def validateZoom(min, max, step) :

    if isEmpty(min) or isEmpty(max) or isEmpty(step) :
        tk.Tk().withdraw()
        ret = messagebox.showinfo('エラー', '縮小拡大の値を入力してください。')
        return False

    if int(min) < 10 or int(max) > 500 :
        tk.Tk().withdraw()
        ret = messagebox.showinfo('エラ', '縮小拡大は10%以上500%以下にしてください。')
        return False

    if int(min) >= int(max) :
        tk.Tk().withdraw()
        ret = messagebox.showinfo('エラ', '縮小拡大は、最小＜最大にしてください。')
        return False

    if not validateStep(step) :
        return False

    return True

###########################
# 引き伸ばし（左右）
#   min   最小(%)
#   max   最大(%)
#   step  ステップ
# 戻り値 True 入力可、False 入力不可
###########################
def validateHorZoom(min, max, step) :

    if isEmpty(min) or isEmpty(max) or isEmpty(step) :
        tk.Tk().withdraw()
        ret = messagebox.showinfo('エラー', '左右引き伸ばし範囲の値を入力してください。')
        return False

    if int(min) < 10 or int(max) > 500 :
        tk.Tk().withdraw()
        ret = messagebox.showinfo('エラ', '左右引き伸ばしは10%以上500%以下にしてください。')
        return False

    if int(min) >= int(max) :
        tk.Tk().withdraw()
        ret = messagebox.showinfo('エラ', '左右引き伸ばしは、最小＜最大にしてください。')
        return False

    if not validateStep(step) :
        return False

    return True

###########################
# 引き伸ばし（上下）
#   min   最小(%)
#   max   最大(%)
#   step  ステップ
# 戻り値 True 入力可、False 入力不可
###########################
def validateVerZoom(min, max, step) :

    if isEmpty(min) or isEmpty(max) or isEmpty(step) :
        tk.Tk().withdraw()
        ret = messagebox.showinfo('エラー', '上下引き伸ばし範囲の値を入力してください。')
        return False

    if int(min) < 10 or int(max) > 500 :
        tk.Tk().withdraw()
        ret = messagebox.showinfo('エラ', '上下引き伸ばしは10%以上500%以下にしてください。')
        return False

    if int(min) >= int(max) :
        tk.Tk().withdraw()
        ret = messagebox.showinfo('エラ', '上下引き伸ばしは、最小＜最大にしてください。')
        return False

    if not validateStep(step) :
        return False

    return True

###########################
# 効果がチェックされた場合
#   効果の値を入力できるようにする。
###########################
def cmdEffect(self) :
    global txtEffectValue
    
    if len(cmbEffect.get()) > 0 :
        if cmbEffect.get() == 'Gray' :
            txtEffectValue['state'] = tk.DISABLED
        else :
            txtEffectValue['state'] = tk.NORMAL
    else :
        txtEffectValue['state'] = tk.DISABLED

###########################
# チェックボックスコマンド
#   bln   = 呼び出し時のオブジェクト(boolean)
#   txt1  = 呼び出し時のオブジェクト(text)
#   txt2  = 呼び出し時のオブジェクト(text)
#   txt3p = 呼び出し時のオブジェクト(text)
###########################
def editText(param) :
    bln, txt1, txt2, txt3 = param
    if bln.get() :
        txt1['state'] = tk.NORMAL
        txt2['state'] = tk.NORMAL
        txt3['state'] = tk.NORMAL
    else :
        txt1['state'] = tk.DISABLED
        txt2['state'] = tk.DISABLED
        txt3['state'] = tk.DISABLED

###########################
# オプション項目表示
#   y      = 高さ
#   txt    = ラベルの文字
#   defMin = デフォルト(最小)
#   defMax = デフォルト(最大)
#   defStep= デフォルト(ステップ)
# 戻り値 チェックボックス、最小、最大、ステップのオブジェクト
###########################
def drawOption(y, txt, defMin, defMax, defStep) :

    #表示位置
    posLeft       = 20   # 左端
    posMin        = 60   # 最小
    posMax        = 140  # 最大
    posRangeLabel = 100  # ～
    posStep       = 200  # ステップなど

    items = [None, None, None, None]

    # 項目表示
    bln = tk.BooleanVar()
    bln.set(True)
    chk = tk.Checkbutton(root, variable=bln, command=partial(editText, items), text=txt)
    chk.place(x=posLeft, y=y)
    items[0] = bln

    txtMin = tk.Entry(width=4, validatecommand=(valDegit, '%P'), validate='all',)
    txtMin.place(x=posMin, y=y+30)
    txtMin.insert(0, defMin)
    items[1] = txtMin

    lbl = tk.Label(text='～')
    lbl.place(x=posRangeLabel, y=y+30)

    txtMax = tk.Entry(width=4, validatecommand=(valDegit, '%P'), validate='all',)
    txtMax.place(x=posMax, y=y+30)
    txtMax.insert(0, defMax)
    items[2] = txtMax

    txtStep = tk.Entry(width=4, validatecommand=(valDegit, '%P'), validate='all',)
    txtStep.place(x=posStep, y=y+30)
    txtStep.insert(0, defStep)
    items[3] = txtStep

    return items

###########################
# 開始処理
#   btnFileRead = ファイル読み込みボタン
#   btnEnd      = 終了ボタン
###########################
def imageTransformStart(btnFileRead, btnEnd) :
    btnFileRead['state'] = tk.DISABLED
    btnEnd['state'] = tk.DISABLED

###########################
# 終了処理
#   btnFileRead = ファイル読み込みボタン
#   btnEnd      = 終了ボタン
###########################
def imageTransformEnd(btnFileRead, btnEnd) :
    btnFileRead['state'] = tk.NORMAL
    btnEnd['state'] = tk.NORMAL

###########################
# 画像変換メイン
###########################
def transformate() :
    global defaultDir
    imageTransformManager = imagetransformmanager.ImageTransformManager()

    # パラメータ設定
    # デフォルトのフォルダ
    imageTransformManager.setDefaultDir(defaultDir)

    # 画像形式選択
    imageTransformManager.setParamCodec(vCodec.get())
    
    # 背景色(最初の一文字を送る)
    imageTransformManager.setParamBack(vBack.get()[0])

    # 効果(最初の一文字を送る)
    if len(vEffect.get()) > 0 :
        # 選択がある場合はチェック
        if not validateEffect(txtEffectValue.get()) :
            return
        imageTransformManager.setParamEffect(vEffect.get()[0])
        imageTransformManager.setEffectValue(int(txtEffectValue.get()))
    else :
        imageTransformManager.setParamEffect('')

    # 回転
    if blnRotate.get() :
        if not validateRotate(txtRotateMin.get(), txtRotateMax.get(), txtRotateStep.get()) :
            return
        imageTransformManager.setRotateOption(blnRotate.get())
        imageTransformManager.setRotateMax(int(txtRotateMax.get()))
        imageTransformManager.setRotateMin(int(txtRotateMin.get()))
        imageTransformManager.setRotateStep(int(txtRotateStep.get()))

    # スキュー
    if blnSkew.get() :
        if not validateSkew(txtSkewMin.get(), txtSkewMax.get(), txtSkewStep.get()) :
            return
        imageTransformManager.setSkewOption(blnSkew.get())
        imageTransformManager.setSkewMax(int(txtSkewMax.get()))
        imageTransformManager.setSkewMin(int(txtSkewMin.get()))
        imageTransformManager.setSkewStep(int(txtSkewStep.get()))
    
    # 水平移動
    if blnHorShift.get() :
        if not validateHorShift(txtHorShiftMin.get(), txtHorShiftMax.get(), txtHorShiftStep.get()) :
            return
        imageTransformManager.setHorShiftOption(blnHorShift.get())
        imageTransformManager.setHorShiftMax(int(txtHorShiftMax.get()))
        imageTransformManager.setHorShiftMin(int(txtHorShiftMin.get()))
        imageTransformManager.setHorShiftStep(int(txtHorShiftStep.get()))

    # 垂直移動
    if blnVerShift.get() :
        if not validateVerShift(txtVerShiftMin.get(), txtVerShiftMax.get(), txtVerShiftStep.get()) :
            return
        imageTransformManager.setVerShiftOption(blnVerShift.get())
        imageTransformManager.setVerShiftMax(int(txtVerShiftMax.get()))
        imageTransformManager.setVerShiftMin(int(txtVerShiftMin.get()))
        imageTransformManager.setVerShiftStep(int(txtVerShiftStep.get()))

    # 縮小拡大
    if blnZoom.get() :
        if not validateZoom(txtZoomMin.get(), txtZoomMax.get(), txtZoomStep.get()) :
            return
        imageTransformManager.setZoomOption(blnZoom.get())
        imageTransformManager.setZoomMax(int(txtZoomMax.get()))
        imageTransformManager.setZoomMin(int(txtZoomMin.get()))
        imageTransformManager.setZoomStep(int(txtZoomStep.get()))

    # 左右引き伸ばし
    if blnHorZoom.get() :
        if not validateHorZoom(txtHorZoomMin.get(), txtHorZoomMax.get(), txtHorZoomStep.get()) :
            return
        imageTransformManager.setHorZoomOption(blnHorZoom.get())
        imageTransformManager.setHorZoomMax(int(txtHorZoomMax.get()))
        imageTransformManager.setHorZoomMin(int(txtHorZoomMin.get()))
        imageTransformManager.setHorZoomStep(int(txtHorZoomStep.get()))

    # 上下引き伸ばし
    if blnVerZoom.get() :
        if not validateVerZoom(txtVerZoomMin.get(), txtVerZoomMax.get(), txtVerZoomStep.get()) :
            return
        imageTransformManager.setVerZoomOption(blnVerZoom.get())
        imageTransformManager.setVerZoomMax(int(txtVerZoomMax.get()))
        imageTransformManager.setVerZoomMin(int(txtVerZoomMin.get()))
        imageTransformManager.setVerZoomStep(int(txtVerZoomStep.get()))

    # 開始処理
    imageTransformStart(btnFileRead, btnEnd)

    # 制御呼び出し
    imageTransformManager.transformate()
    
    #終了処理
    defaultDir = imageTransformManager.getDefaultDir()
    imageTransformEnd(btnFileRead, btnEnd)

    print('FINISHED.')


###########################
# メイン処理
###########################

# メイン画面とボタン
root.geometry('280x600')
root.title('画像変換')

# 数値検証
valDegit   = root.register(validateDegit)

#表示位置
posLeft       = 20   # 左端

# ファイル選択ボタン
btnFileRead = tk.Button(root, text='元画像フォルダ選択', command=transformate)
btnFileRead.place(x=posLeft, y=10)

# 終了ボタン
btnEnd = tk.Button(root, text='終了', command=root.quit)
btnEnd.place(x=220, y=10)

# 画像形式選択
lblCodec = tk.Label(text='画像形式')
lblCodec.place(x=posLeft, y=50)
mCodec = ['', 'jpg', 'png','bmp']
vCodec = tk.StringVar()
cmbCodec = ttk.Combobox(root, width=10, textvariable= vCodec, values=mCodec)
cmbCodec.set(mCodec[0])
cmbCodec.pack(pady=8)
cmbCodec.place(x = 120, y = 50)

# 背景色選択
lblBack = tk.Label(text='背景色')
lblBack.place(x=posLeft, y=80)
mBack = ['Random', 'White', 'Black']
vBack = tk.StringVar()
cmbBack = ttk.Combobox(root, width=10, textvariable= vBack, values=mBack)
cmbBack.set(mBack[0])
cmbBack.pack(pady=8)
cmbBack.place(x = 120, y = 80)

# 効果
lblEffect = tk.Label(text='効果')
lblEffect.place(x=posLeft, y=110)
mEffect = ['', 'Blur', 'Gray', 'Noise']
vEffect = tk.StringVar()
cmbEffect = ttk.Combobox(root, width=10, textvariable= vEffect, values=mEffect)
cmbEffect.set(mEffect[0])
cmbEffect.pack(pady=8)
cmbEffect.place(x =120, y = 110)
cmbEffect.bind("<<ComboboxSelected>>", cmdEffect)
txtEffectValue = tk.Entry(width=4, validatecommand=(valDegit, '%P'), validate='all',)
txtEffectValue.place(x=220, y=110)
txtEffectValue.insert(0, '20')
txtEffectValue['state'] = tk.DISABLED


# 回転範囲
blnRotate, txtRotateMin, txtRotateMax, txtRotateStep = drawOption( 
                           150, '回転 角度範囲(左周り、右角度°～左角度°)', '60',  '60',  '10' )

# スキュー範囲,ステップ
blnSkew, txtSkewMin, txtSkewMax, txtSkewStep = drawOption( 
                           210,  '傾斜角度 範囲(右角度°～左角度°)',  '30', '30', '10' )

# 左右シフト範囲,ステップ
blnHorShift, txtHorShiftMin, txtHorShiftMax, txtHorShiftStep = drawOption( 
                           270, '左右シフト 範囲(左%～右%)', '20', '20', '10' )

# 左右シフト範囲,ステップ
blnVerShift, txtVerShiftMin, txtVerShiftMax, txtVerShiftStep = drawOption( 
                           330, '上下シフト 範囲(上%～下%)',  '20', '20', '10' )

# 縮小拡大範囲,ステップ
blnZoom, txtZoomMin, txtZoomMax, txtZoomStep = drawOption( 
                           390, '縮小拡大 範囲(最小%～最大%)', '70',  '130', '10' )

# 左右縮小引き伸ばし範囲,ステップ
blnHorZoom, txtHorZoomMin, txtHorZoomMax, txtHorZoomStep = drawOption( 
                           450, '左右縮小引き伸ばし 範囲(最小%～最大%)', '70',  '130', '10' )

# 上下縮小引き伸ばし範囲,ステップ
blnVerZoom, txtVerZoomMin, txtVerZoomMax, txtVerZoomStep = drawOption( 
                           510, '上下縮小引き伸ばし 範囲(最小%～最大%)', '70',  '130', '10' )

# 入力メインループ
root.mainloop()

print('END.')
