#######################################################
# 画像の縮小拡大
# (全体の縮小拡大、水平引き伸ばし、垂直引き伸ばし)
#
# Shin.Onda 2023/08//26 all rights reserved.
#
#######################################################
import re
import numpy as np
import random
import cv2
from createcanvas import CreateCanvas

class ImageZoom :

    ######################################
    # 縮小拡大(全体、水平、垂直)
    # パラメータ
    #   paraImg = イメージ
    #   imgBack = 背景('R':乱数, 'W':白, 'B':黒)
    #   magSmall = 拡大縮小　最小％
    #   magLarge = 拡大縮小　最大％
    #   magStep  = 間隔
    #   mode     = 0:全体、1:水平、2:垂直
    # 戻り値 イメージリスト
    ######################################
    def zoom(self, paramImg, imgBack='R', magSmall=50, magLarge=150, magStep=10, mode=0) :
        # 画像パラメータ
        orgH, orgW, orgC = paramImg.shape

        # 縮小拡大のループ
        mag = 1.0
        imgList = []
        for i in range(0, magStep) :
            canvas = paramImg.copy()

            # 倍率に合わせたサイズ変更
            if magStep > 1 :
                delta = (magLarge - magSmall) / (magStep-1)
                mag   = (magSmall + (delta * i)) / 100.0
            else :
                mag   = magSmall / 100.0
            print('Zoom {0}_{1:06.4f}'.format(mode, mag))

            newImg = None
            if mode == 1 :
               # 水平
               newImg = cv2.resize(canvas, dsize=(int(mag * orgW), orgH))
            elif mode == 2:
               # 垂直
               newImg = cv2.resize(canvas, dsize=(orgW, int(mag * orgH)))
            else :
               # 全体
               newImg = cv2.resize(canvas, dsize=(int(mag * orgW), int(mag * orgH)))
            newH, newW, newC = newImg.shape

            if mag > 1 :
                # 拡大
                sx = int((newW - orgW) / 2)
                ex = sx + orgW
                sy = int((newH - orgH) / 2)
                ey = sy + orgH
                newCanvas = newImg[sy:ey, sx:ex]
            else :
                # 縮小
                sx = int((orgW - newW) / 2)
                ex = sx + newW
                sy = int((orgH - newH) / 2)
                ey = sy + newH
                newCanvas = CreateCanvas.create(orgH, orgW, imgBack)
                newCanvas[sy:ey, sx:ex] = newImg
            # 戻り値リストに追加
            imgList.append(newCanvas)
        
        return imgList

